/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Future;

import com.koutra.dist.proc.model.IFaucet;
import com.koutra.dist.proc.model.ISink;
import com.koutra.dist.proc.util.ExecutorsHelper;

/**
 * This class is used by the KoutraXform designer in order to launch a new
 * transformation pipeline in a separate process. It reads the serialized pipeline
 * from a temporary file and consumes all the sinks asynchronously.
 * 
 * @author Pafsanias Ftakas
 */
public class ProcessExecutor {

	public static void main(String[] args) {
		try {
			if (args.length < 1) {
				System.out.println("No process stream has been specified.");
				return;
			}
			
			String sourceStream = args[0];
			List<ISink> sinkList = new ArrayList<ISink>();
			
			DataInputStream in = new DataInputStream(new FileInputStream(sourceStream));
			while (true) {
				String clazzName = null;
				try {
					clazzName = in.readUTF();
				} catch (EOFException e) {
					break;
				}
				
				Class<?> clazz;
				try {
					clazz = Class.forName(clazzName);
				} catch (ClassNotFoundException e) {
					throw new InstantiationException("Unable to load class: " + clazzName);
				}
				IFaucet faucet = (IFaucet) clazz.newInstance();
				faucet.readFrom(in);
				sinkList.addAll(faucet.getTerminalSinks());
			}
			in.close();
			
			File f = new File(sourceStream);
			f.delete();
			
			List<Future<?>> futures = new ArrayList<Future<?>>();
			for (ISink sink : sinkList) {
				futures.add(sink.consumeAsynchronously());
			}
			
			for (Future<?> future : futures) {
				future.get();
			}
			
			for (ISink sink : sinkList) {
				sink.dispose();
			}
			
			ExecutorsHelper.getInstance().shutdown();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
