/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.cluster;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.jgroups.util.Streamable;

/**
 * Representation of the request from a cluster executor service to the cluster of
 * cooperating nodes.
 * 
 * @author Pafsanias Ftakas
 */
public class Request implements Streamable {
	
	public enum Type {
		EXECUTE,
		RESULT
	}
	
	private Type type;
	private Streamable payload;
	private TaskID taskID;
	private Streamable result;
	
	/**
	 * @deprecated use an initializing constructor.
	 */
	public Request() {
		this.type = null;
		this.payload = null;
		this.taskID = null;
		this.result = null;
	}
	
	public Request(Type type, Streamable payload, TaskID taskID, Streamable result) {
		this.type = type;
		this.payload = payload;
		this.taskID = taskID;
		this.result = result;
	}

	public Type getType() {
		return type;
	}

	public Streamable getPayload() {
		return payload;
	}

	public TaskID getTaskID() {
		return taskID;
	}

	public Streamable getResult() {
		return result;
	}

	@Override
	public void readFrom(DataInputStream in) throws IOException,
			IllegalAccessException, InstantiationException {
		type = Type.valueOf(in.readUTF());
		String clazz = in.readUTF();
		if (!"null".equals(clazz)) {
			try {
				payload = (Streamable) Class.forName(clazz).newInstance();
			} catch (Exception e) {
				throw new InstantiationException("Unable to create instance of: " +
						clazz + " because of: " + e.getMessage());
			}
			payload.readFrom(in);
		} else {
			payload = null;
		}
		taskID = TaskID.createTaskIDFromStream(in);
		clazz = in.readUTF();
		if (!"null".equals(clazz)) {
			try {
				result = (Streamable) Class.forName(clazz).newInstance();
			} catch (Exception e) {
				throw new InstantiationException("Unable to create instance of: " +
						clazz + " because of: " + e.getMessage());
			}
			result.readFrom(in);
		} else {
			result = null;
		}
	}

	@Override
	public void writeTo(DataOutputStream out) throws IOException {
		out.writeUTF(type.name());
		if (payload == null) {
			out.writeUTF("null");
		} else {
			out.writeUTF(payload.getClass().getCanonicalName());
			payload.writeTo(out);
		}
		taskID.writeTo(out);
		if (result == null) {
			out.writeUTF("null");
		} else {
			out.writeUTF(result.getClass().getCanonicalName());
			result.writeTo(out);
		}
	}

}
