/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.xml;

import java.io.IOException;
import java.io.Reader;

import org.apache.log4j.Logger;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;

import com.koutra.dist.proc.pipeline.ReaderToXMLDemuxPipelineItem;

public class ReaderSAXEventReader implements XMLReader {
	private static final Logger logger = Logger.getLogger(ReaderSAXEventReader.class);
	
	private class DocumentLocator implements Locator {

		public int getColumnNumber() {
			return -1;
		}

		public int getLineNumber() {
			return -1;
		}

		public String getPublicId() {
			return null;
		}

		public String getSystemId() {
			return null;
		}
		
	}
	
	protected ContentHandler contentHandler;
	protected Reader reader;
	protected ReaderToXMLDemuxPipelineItem demux;
	protected IReaderSAXCallback callback;
	
	public ReaderSAXEventReader(Reader reader,
			ReaderToXMLDemuxPipelineItem demux,
			IReaderDemuxSAXCallback callback) {
		this.contentHandler = null;
		this.reader = reader;
		this.demux = demux;
		this.callback = callback;
	}
	
	public ReaderSAXEventReader(Reader reader, IReaderSAXCallback callback) {
		this.contentHandler = null;
		this.reader = reader;
		this.demux = null;
		this.callback = callback;
	}
	
	protected void convertReader() throws SAXException, IOException {
		contentHandler.startDocument();
		contentHandler.startElement("", "result", "result", new AttributesImpl());

		int count;
		char[] buffer = new char[8 * 1024];
		while ((count = reader.read(buffer)) != -1) {
			callback.transformBuffer(buffer, 0, count, contentHandler);
			if (logger.isTraceEnabled()) {
				String readInput = new String(buffer, 0, count);
				logger.trace("Read " + readInput + " from reader " + reader);
			}
			if (demux != null) {
				while (((IReaderDemuxSAXCallback) callback).shouldChangePipeline()) {
					contentHandler.endElement("", "result", "result");
					contentHandler.endDocument();
					
					if (logger.isTraceEnabled())
						logger.trace("Starting pipeline switch");
					demux.switchPipeline();
					if (logger.isTraceEnabled())
						logger.trace("Ending pipeline switch");
					
					contentHandler.startDocument();
					contentHandler.startElement("", "result", "result", new AttributesImpl());
					
					callback.transformBuffer(buffer, 0, 0, contentHandler);
				}
			}
		}
		callback.transformBuffer(buffer, 0, count, contentHandler);
		
		contentHandler.endElement("", "result", "result");
		contentHandler.endDocument();
		
		if (demux != null) {
			demux.signalTermination();
		}
	}

	public void parse(InputSource input) throws IOException, SAXException {
		contentHandler.setDocumentLocator(new DocumentLocator());
		try {
			convertReader();
		} catch (IOException e) {
			throw new SAXException("Unable to convert result set", e);
		}
	}

	public void parse(String systemId) throws IOException, SAXException {
		contentHandler.setDocumentLocator(new DocumentLocator());
		try {
			convertReader();
		} catch (IOException e) {
			throw new SAXException("Unable to convert result set", e);
		}
	}

	public ContentHandler getContentHandler() {
		return contentHandler;
	}

	public void setContentHandler(ContentHandler handler) {
		contentHandler = handler;
		if (logger.isTraceEnabled())
			logger.trace("Switched content handler to: " + contentHandler);
	}

	//////////////////////////////////////////////////////////////////////////////////////
	// The rest of the implementation is not used.
	//////////////////////////////////////////////////////////////////////////////////////
	
	public DTDHandler getDTDHandler() {
		return null;
	}

	public EntityResolver getEntityResolver() {
		return null;
	}

	public ErrorHandler getErrorHandler() {
		return null;
	}

	public boolean getFeature(String name) throws SAXNotRecognizedException,
			SAXNotSupportedException {
		return false;
	}

	public Object getProperty(String name) throws SAXNotRecognizedException,
			SAXNotSupportedException {
		return null;
	}

	public void setDTDHandler(DTDHandler handler) {
	}

	public void setEntityResolver(EntityResolver resolver) {
	}

	public void setErrorHandler(ErrorHandler handler) {
	}

	public void setFeature(String name, boolean value)
			throws SAXNotRecognizedException, SAXNotSupportedException {
	}

	public void setProperty(String name, Object value)
			throws SAXNotRecognizedException, SAXNotSupportedException {
	}

}
